@@ -4,7 +4,7 @@
  *   License:	LGPL - See file COPYING.LIB for details.
  *   Author:	Stefan Hundhammer <sh@suse.de>
  *
- *   Updated:	2003-01-30
+ *   Updated:	2006-02-04
  */
 
 
@@ -24,10 +24,12 @@
 #include <kiconloader.h>
 
 #include "kdirtreeview.h"
+#include "kdirreadjob.h"
 #include "kdirtreeiterators.h"
 #include "kpacman.h"
 
 #define SEPARATE_READ_JOBS_COL	0
+#define VERBOSE_PROGRESS_INFO	0
 
 using namespace KDirStat;
 
@@ -100,6 +102,7 @@ KDirTreeView::KDirTreeView( QWidget * parent )
     _blockDevIcon	= loadIcon( "blockdevice"	);
     _charDevIcon	= loadIcon( "chardevice"	);
     _fifoIcon		= loadIcon( "socket"		);
+    _stopIcon		= loadIcon( "stop"		);
     _readyIcon		= QPixmap();
 
 #undef loadIcon
@@ -123,6 +126,8 @@ KDirTreeView::KDirTreeView( QWidget * parent )
 
    _contextInfo	  = new QPopupMenu;
    _idContextInfo = _contextInfo->insertItem ( "dummy" );
+
+   createTree();
 }
 
 
@@ -184,7 +189,7 @@ KDirTreeView::idleDisplay()
 	// A pathological case: The user requested sorting by read jobs, and
 	// now that everything is read, the items are still in that sort order.
 	// Not only is that sort order now useless (since all read jobs are
-	// done), it is contrary to the (now changed) semanantics of this
+	// done), it is contrary to the (now changed) semantics of this
 	// column. Calling QListView::sort() might do the trick, but we can
 	// never know just how clever that QListView widget tries to be and
 	// maybe avoid another sorting by the same column - so let's use the
@@ -195,12 +200,26 @@ KDirTreeView::idleDisplay()
 	setSorting( _percentNumCol );
     }
 #endif
+
     _readJobsCol = -1;
 }
 
 
 void
 KDirTreeView::openURL( KURL url )
+{
+    clear();
+    _tree->clear();
+
+    // Implicitly calling prepareReading() via the tree's startingReading() signal
+    _tree->startReading( url );
+
+    logActivity( 30 );
+}
+
+
+void
+KDirTreeView::createTree()
 {
     // Clean up any old leftovers
 
@@ -233,6 +252,9 @@ KDirTreeView::openURL( KURL url )
     connect( _tree, SIGNAL( finished()     ),
 	     this,  SLOT  ( slotFinished() ) );
 
+    connect( _tree, SIGNAL( aborted()     ),
+	     this,  SLOT  ( slotAborted() ) );
+
     connect( _tree, SIGNAL( finalizeLocal( KDirInfo * ) ),
 	     this,  SLOT  ( finalizeLocal( KDirInfo * ) ) );
 
@@ -241,18 +263,12 @@ KDirTreeView::openURL( KURL url )
 
     connect( _tree, SIGNAL( selectionChanged( KFileInfo * ) ),
 	     this,  SLOT  ( selectItem      ( KFileInfo * ) ) );
-
-    // Implicitly calling prepareReading() via the tree's startingReading() signal
-    _tree->startReading( url );
-
-    logActivity( 30 );
 }
 
 
 void
 KDirTreeView::prepareReading()
 {
-
     // Prepare cyclic update
 
     if ( _updateTimer )
@@ -277,9 +293,6 @@ KDirTreeView::prepareReading()
     busyDisplay();
     emit startingReading();
 
-
-    // Actually do something
-
     _stopWatch.start();
 }
 
@@ -309,6 +322,14 @@ KDirTreeView::refreshSelected()
 }
 
 
+void
+KDirTreeView::abortReading()
+{
+    if ( _tree )
+	_tree->abortReading();
+}
+
+
 void
 KDirTreeView::clear()
 {
@@ -320,6 +341,26 @@ KDirTreeView::clear()
 }
 
 
+bool
+KDirTreeView::writeCache( const QString & cacheFileName )
+{
+    if ( _tree )
+	return _tree->writeCache( cacheFileName );
+
+    return false;
+}
+
+
+void
+KDirTreeView::readCache( const QString & cacheFileName )
+{
+    clear();
+    _tree->clear();
+
+    _tree->readCache( cacheFileName );
+}
+
+
 void
 KDirTreeView::addChild( KFileInfo *newChild )
 {
@@ -431,6 +472,18 @@ KDirTreeView::slotFinished()
     updateSummary();
     logActivity( 30 );
 
+    if ( _tree->root() &&
+	 _tree->root()->totalSubDirs() == 0 &&	// No subdirs
+	 _tree->root()->totalItems() > 0 )	// but file children
+    {
+	QListViewItem * root = firstChild();
+
+	if ( root )
+	    root->setOpen( true );
+    }
+
+
+
 #if 0
     for ( int i=0; i < DEBUG_COUNTERS; i++ )
     {
@@ -445,6 +498,25 @@ KDirTreeView::slotFinished()
 }
 
 
+void
+KDirTreeView::slotAborted()
+{
+    emit progressInfo( i18n( "Aborted. Elapsed time: %1" )
+		       .arg( formatTime( _stopWatch.elapsed(), true ) ) );
+
+    if ( _updateTimer )
+    {
+	delete _updateTimer;
+	_updateTimer = 0;
+    }
+
+    idleDisplay();
+    updateSummary();
+
+    emit aborted();
+}
+
+
 void
 KDirTreeView::finalizeLocal( KDirInfo *dir )
 {
@@ -464,19 +536,15 @@ KDirTreeView::sendProgressInfo( const QString & newCurrentDir )
 {
     _currentDir = newCurrentDir;
 
+#if VERBOSE_PROGRESS_INFO
     emit progressInfo( i18n( "Elapsed time: %1   reading directory %2" )
 		       .arg( formatTime( _stopWatch.elapsed() ) )
 		       .arg( _currentDir ) );
-}
-
-
-#if QT_VERSION < 300
-void
-KDirTreeView::sendProgressInfo()
-{
-    sendProgressInfo( _currentDir );
-}
+#else
+    emit progressInfo( i18n( "Elapsed time: %1" )
+		       .arg( formatTime( _stopWatch.elapsed() ) ) );
 #endif
+}
 
 
 KDirTreeViewItem *
@@ -597,8 +665,8 @@ KDirTreeView::fillColor( int level ) const
 {
     if ( level < 0 )
     {
-	level = 0;
 	kdWarning() << k_funcinfo << "Invalid argument: " << level << endl;
+	level = 0;
     }
 
     return _fillColor [ level % _usedFillColors ];
@@ -740,7 +808,36 @@ KDirTreeView::popupContextMenu( QListViewItem *	listViewItem,
 
     if ( column == _ownSizeCol && ! item->orig()->isDotEntry() )
     {
-	popupContextSizeInfo( pos, item->orig()->size() );
+	KFileInfo * orig = item->orig();
+
+	if ( orig->isSparseFile() || ( orig->links() > 1 && orig->isFile() ) )
+	{
+	    QString text;
+
+	    if ( orig->isSparseFile() )
+	    {
+		text = i18n( "Sparse file: %1 (%2 Bytes) -- allocated: %3 (%4 Bytes)" )
+		    .arg( formatSize( orig->byteSize() ) )
+		    .arg( formatSizeLong( orig->byteSize()  ) )
+		    .arg( formatSize( orig->allocatedSize() ) )
+		    .arg( formatSizeLong( orig->allocatedSize() ) );
+	    }
+	    else
+	    {
+		text = i18n( "%1 (%2 Bytes) with %3 hard links => effective size: %4 (%5 Bytes)" )
+		    .arg( formatSize( orig->byteSize() ) )
+		    .arg( formatSizeLong( orig->byteSize() ) )
+		    .arg( orig->links() )
+		    .arg( formatSize( orig->size() ) )
+		    .arg( formatSizeLong( orig->size() ) );
+	    }
+
+	    popupContextInfo( pos, text );
+	}
+	else
+	{
+	    popupContextSizeInfo( pos, orig->size() );
+	}
     }
 
     if ( column == _totalSizeCol &&
@@ -765,11 +862,17 @@ void
 KDirTreeView::popupContextSizeInfo( const QPoint &	pos,
 				    KFileSize		size )
 {
-    QString info = formatSizeLong( size ) + " " + i18n( "Bytes" );
+    QString info;
 
-    if ( size > 1024 )
+    if ( size < 1024 )
     {
-	info += " (" + formatSize( size ) + ")";
+	info = formatSizeLong( size ) + " " + i18n( "Bytes" );
+    }
+    else
+    {
+	info = i18n( "%1 (%2 Bytes)" )
+	    .arg( formatSize( size ) )
+	    .arg( formatSizeLong( size ) );
     }
 
     popupContextInfo( pos, info );
@@ -871,7 +974,7 @@ KDirTreeView::sendMailToOwner()
 	return;
     }
 
-    QString owner = KAnyDirReadJob::owner( fixedUrl( _selection->orig()->url() ) );
+    QString owner = KioDirReadJob::owner( fixedUrl( _selection->orig()->url() ) );
     QString subject = i18n( "Disk Usage" );
     QString body =
 	i18n("Please check your disk usage and clean up if you can. Thank you." )
@@ -950,7 +1053,39 @@ KDirTreeViewItem::init( KDirTreeView *		view,
 
 	if ( ! _orig->isDevice() )
 	{
-	    setText( view->ownSizeCol(), formatSize( _orig->size() ) );
+	    QString text;
+
+	    if ( _orig->isFile() && ( _orig->links() > 1 ) ) // Regular file with multiple links
+	    {
+		if ( _orig->isSparseFile() )
+		{
+		    text = i18n( "%1 / %2 Links (allocated: %3)" )
+			.arg( formatSize( _orig->byteSize() ) )
+			.arg( formatSize( _orig->links() ) )
+			.arg( formatSize( _orig->allocatedSize() ) );
+		}
+		else
+		{
+		    text = i18n( "%1 / %2 Links" )
+			.arg( formatSize( _orig->byteSize() ) )
+			.arg( _orig->links() );
+		}
+	    }
+	    else // No multiple links or no regular file
+	    {
+		if ( _orig->isSparseFile() )
+		{
+		    text = i18n( "%1 (allocated: %2)" )
+			.arg( formatSize( _orig->byteSize() ) )
+			.arg( formatSize( _orig->allocatedSize() ) );
+		}
+		else
+		{
+		    text = formatSize( _orig->size() );
+		}
+	    }
+
+	    setText( view->ownSizeCol(), text );
 	}
 
 	QListViewItem::setOpen ( _orig->treeLevel() < _view->openLevel() );
@@ -975,8 +1110,9 @@ KDirTreeViewItem::init( KDirTreeView *		view,
 	 * is still being read.
 	 */
 
-	if ( _orig->readState() == KDirQueued ||
-	     _orig->readState() == KDirReading  )
+	if ( _orig->readState() == KDirQueued  ||
+	     _orig->readState() == KDirReading ||
+	     _orig->readState() == KDirCached    )
 	{
 	    setExpandable( true );
 	}
@@ -1016,7 +1152,11 @@ KDirTreeViewItem::setIcon()
     }
     else if ( _orig->isDir() )
     {
-	if ( _orig->readState() == KDirError )
+	if  ( _orig->readState() == KDirAborted )
+	{
+	    icon = _view->stopIcon();
+	}
+	else if ( _orig->readState() == KDirError   )
 	{
 	    icon = _view->unreadableDirIcon();
 	    setExpandable( false );
@@ -1055,9 +1195,14 @@ KDirTreeViewItem::updateSummary()
 
     if ( _orig->isDir() || _orig->isDotEntry() )
     {
-	setText( _view->totalSizeCol(),		" " + formatSize(  _orig->totalSize()	) );
-	setText( _view->totalItemsCol(),	" " + formatCount( _orig->totalItems()	) );
-	setText( _view->totalFilesCol(),	" " + formatCount( _orig->totalFiles()	) );
+	QString prefix = " ";
+
+	if ( _orig->readState() == KDirAborted )
+	    prefix = " >";
+
+	setText( _view->totalSizeCol(),		prefix + formatSize(  _orig->totalSize()	) );
+	setText( _view->totalItemsCol(),	prefix + formatCount( _orig->totalItems()	) );
+	setText( _view->totalFilesCol(),	prefix + formatCount( _orig->totalFiles()	) );
 
 	if ( _view->readJobsCol() >= 0 )
 	{
@@ -1321,7 +1466,7 @@ KDirTreeViewItem::findDotEntry() const
     {
 	if ( child->orig()->isDotEntry() )
 	    return child;
-	
+
 	child = child->next();
     }
 
