@@ -4,7 +4,7 @@
  *   License:	LGPL - See file COPYING.LIB for details.
  *   Author:	Stefan Hundhammer <sh@suse.de>
  *
- *   Updated:	2003-01-30
+ *   Updated:	2006-01-06
  */
 
 
@@ -117,7 +117,7 @@ namespace KDirStat
 	/**
 	 * Enable / disable PacMan animation in this tree view during directory
 	 * reading. This is disabled by default since it eats quite some
-	 * performance. 
+	 * performance.
 	 **/
 	void	enablePacManAnimation( bool enable ) { _doPacManAnimation = enable; }
 	/**
@@ -156,7 +156,7 @@ namespace KDirStat
 	 * Set all tree colors to default values.
 	 **/
 	void setDefaultFillColors();
-	
+
 	/**
 	 * Set the number of used percentage bar fill colors
 	 * (1..KDirTreeViewMaxFillColor).
@@ -245,6 +245,7 @@ namespace KDirStat
 	QPixmap blockDevIcon()		const 	{ return _blockDevIcon;		}
 	QPixmap charDevIcon()		const 	{ return _charDevIcon;		}
 	QPixmap fifoIcon()		const 	{ return _fifoIcon;		}
+	QPixmap stopIcon()		const 	{ return _stopIcon;		}
 	QPixmap	workingIcon()		const	{ return _workingIcon;		}
 	QPixmap	readyIcon()		const	{ return _readyIcon;		}
 
@@ -259,7 +260,7 @@ namespace KDirStat
 	 **/
 	void	incDebugCount( int i );
 
-	
+
     public slots:
 
 	/**
@@ -277,6 +278,11 @@ namespace KDirStat
 	 **/
 	void refreshSelected();
 
+	/**
+	 * Forcefully stop a running read process.
+	 **/
+	void abortReading();
+
 	/**
 	 * Clear this view's contents.
 	 **/
@@ -308,7 +314,7 @@ namespace KDirStat
 	 * Close all tree branches except the one specified.
 	 **/
 	void closeAllExcept( KDirTreeViewItem *except );
-	
+
 	/**
 	 * Send a standardized mail to the owner of the selected branch.
 	 * The user will get a mailer window where he can edit that mail all he
@@ -318,7 +324,7 @@ namespace KDirStat
 	 * branch on.
 	 **/
 	void sendMailToOwner();
-	
+
 	/**
 	 * Notification of a change in the KDE palette, i.e. the user selected
 	 * and applied different colors in the KDE control center.
@@ -346,7 +352,19 @@ namespace KDirStat
 	 * Reimplemented from QWidget.
 	 **/
 	virtual QSize minimumSizeHint() const { return QSize( 0, 0 ); }
-	
+
+	/**
+	 * Write the current tree to a cache file.
+	 *
+	 * Returns true if OK, false upon error.
+	 **/
+	bool writeCache( const QString & cacheFileName );
+
+	/**
+	 * Read a cache file.
+	 **/
+	void readCache( const QString & cacheFileName );
+
 
     protected slots:
 
@@ -374,6 +392,12 @@ namespace KDirStat
 	 **/
 	void	slotFinished();
 
+	/**
+	 * Signal abortion of all read jobs, finalize display and terminate pending
+	 * cyclic visual update.
+	 **/
+	void	slotAborted();
+
 	/**
 	 * Signal end of one read job at this level and finalize display of
 	 * this level.
@@ -434,12 +458,10 @@ namespace KDirStat
 	void 	popupContextInfo	( const QPoint &	pos,
 					  const QString & 	info );
 
-	
-    protected slots:
 
 	/**
 	 * Notification that a column has just been resized, thus may need
-	 * repaining. 
+	 * repaining.
 	 **/
         void columnResized( int column, int oldSize, int newSize );
 
@@ -463,6 +485,11 @@ namespace KDirStat
 	 **/
 	void finished();
 
+	/**
+	 * Emitted when reading this tree has been aborted.
+	 **/
+	void aborted();
+
 	/**
 	 * Emitted when the currently selected item changes.
 	 * Caution: 'item' may be 0 when the selection is cleared.
@@ -497,9 +524,18 @@ namespace KDirStat
 	 **/
 	void userActivity( int points );
 
-	
+
     protected:
 
+	/**
+	 * Create a new tree (and delete the old one if there is one)
+	 **/
+	void createTree();
+
+	//
+	// Data members
+	//
+
 	KDirTree *		_tree;
 	QTimer *		_updateTimer;
 	QTime			_stopWatch;
@@ -533,7 +569,7 @@ namespace KDirStat
 	int	_latestMtimeCol;
 	int	_readJobsCol;
 	int	_sortCol;
-	
+
 	int 	_debugCount[ DEBUG_COUNTERS ];
 	QString	_debugFunc [ DEBUG_COUNTERS ];
 
@@ -551,6 +587,7 @@ namespace KDirStat
 	QPixmap _blockDevIcon;
 	QPixmap _charDevIcon;
 	QPixmap _fifoIcon;
+	QPixmap _stopIcon;
 	QPixmap	_workingIcon;
 	QPixmap	_readyIcon;
     };
@@ -680,7 +717,7 @@ namespace KDirStat
 	void openSubtree();
 
 	/**
-	 * Recursively close all tree branches from here on downwards. 
+	 * Recursively close all tree branches from here on downwards.
 	 **/
 	void closeSubtree();
 
@@ -688,7 +725,7 @@ namespace KDirStat
 	 * Close all tree branches except this one from the root on.
 	 **/
 	void closeAllExceptThis();
-	
+
 	/**
 	 * Returns the number of open items in this subtree.
 	 **/
@@ -700,7 +737,7 @@ namespace KDirStat
 	 **/
 	QString asciiDump();
 
-	
+
     protected:
 
 	/**
@@ -727,7 +764,7 @@ namespace KDirStat
 	 **/
 	KDirTreeViewItem * findDotEntry() const;
 
-	
+
 	/**
 	 * Paint method. Reimplemented from @ref QListViewItem so different
 	 * colors can be used - and of course for painting percentage bars.
@@ -801,7 +838,7 @@ namespace KDirStat
 	}
 	else
 	    stream << "<NULL>";
-	
+
 	return stream;
     }
 
@@ -861,7 +898,7 @@ namespace KDirStat
      **/
     QColor contrastingColor ( const QColor &desiredColor,
 			      const QColor &contrastColor );
-    
+
 }	// namespace KDirStat
 
 
