@@ -4,12 +4,10 @@
  *   License:	LGPL - See file COPYING.LIB for details.
  *   Author:	Stefan Hundhammer <sh@suse.de>
  *
- *   Updated:	2003-02-04
+ *   Updated:	2006-02-04
  */
 
 
-#include <sys/stat.h>
-
 #include <qevent.h>
 #include <qregexp.h>
 
@@ -179,6 +177,9 @@ KTreemapView::contentsMousePressEvent( QMouseEvent * event )
 
     KTreemapTile * tile = tileAt( event->pos() );
 
+    if ( ! tile )
+	return;
+
     switch ( event->button() )
     {
 	case LeftButton:
@@ -218,12 +219,12 @@ KTreemapView::contentsMousePressEvent( QMouseEvent * event )
 		    // If a directory (non-leaf tile) is already selected,
 		    // don't override this by
 
-		    emit contextMenu( _selectedTile, event->pos() );
+		    emit contextMenu( _selectedTile, event->globalPos() );
 		}
 		else
 		{
 		    selectTile( tile );
-		    emit contextMenu( tile, event->pos() );
+		    emit contextMenu( tile, event->globalPos() );
 		}
 
 		emit userActivity( 3 );
@@ -245,6 +246,9 @@ KTreemapView::contentsMouseDoubleClickEvent( QMouseEvent * event )
 
     KTreemapTile * tile = tileAt( event->pos() );
 
+    if ( ! tile )
+	return;
+
     switch ( event->button() )
     {
 	case LeftButton:
